/**

        project     PAFEE
 *      @file:     pafee_stdio.c
 *      @author:   Technocrats
 *      @brief:    Structure tool define header

*/

#include <stdarg.h>
/**
 * @brief  %s result
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int ChangeFromS(
	char *pOutputString,         /**< output string */
	const int length,            /**< length of string */
	const int maxLength,         /**< limit length of string */
	const char *pInputString,    /**< %s string */
	const int sign,              /**< %- or %+ */
	const int digit1,            /**< %num non = -1 */
	const int digit2             /**< %.num non = -1*/
);

/**
 * @brief  %x result
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int ChangeFromX(
	char *pOutputString,          /**< output string */
	const int length,             /**< length of string */
	const int maxLength,          /**< limit length of string */
	const long number,            /**< %d integer */
	const int sign,               /**< %- or %+ */
	const int digit1,             /**< %num non = -1 */
	const int digit2,             /**< %.num non = -1*/
	const unsigned char base,     /**< change base */
	const char outCase            /**< case of output character */
);

/**
 * @brief  %c result
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int ChangeFromC(
	char *pOutputString,         /**< output string */
	const int length,            /**< length of string */
	const int maxLength,         /**< limit length of string */
	const char input,            /**< %c character */
	const int sign,              /**< %- or %+ */
	const int digit1             /**< %num non = -1 */
);

/**
 * @brief  %d result
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int ChangeFromD(
	char *pOutputString,          /**< output string */
	const int length,             /**< length of string */
	const int maxLength,          /**< limit length of string */
	const long number,            /**< %d integer */
	const int sign,               /**< %- or %+ */
	const int digit1,             /**< %num non = -1 */
	const int digit2,             /**< %.num non = -1*/
	const unsigned char base      /**< change base */
);

/**
 * @brief  %p result
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int ChangeFromP(
	char *pOutputString,      /**< output string */
	const int length,         /**< length of string */
	const int maxLength,      /**< limit length of string */
	const long number,        /**< %d integer */
	const int sign,           /**< %- or %+ */
	const int digit1,         /**< %num non = -1 */
	const int digit2,         /**< %.num non = -1*/
	const unsigned char base  /**< change base */
);

/**
 * @brief  int to hex
 * @return result
 * @retval if false result=0
 * @retval if true  result=hex character
 */
static char ChangeIntToHex(
	char digit,    /**< hex number */
	char outCase   /**< case of output character */
);

#ifdef PAFEE_LIB_STDIO_ENABLE
/**
 * @brief  stdio::sprintf function
 * @param  convert string
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
int pafee_sprintf(
	char *convertString,
	const char *pFormat,
	...
)
{
	va_list list;
	const char *character;
	char *pOutput;
	char *pString;
	int size;
	long number;
	char byte;
	int sign;
	int digit1;
	int digit2;

	va_start(list, pFormat);
	pOutput = convertString;
	character = pFormat;
	size = 0;

	while(*character != 0)
	{
		if(*character != '%') /* if not % */
		{
			pOutput[size] = *character;
			pOutput[++size] = '\0';

			character++; /* next character */
		}
		else /* % */
		{
			character++; /* next character */

			/* %+ or %- or others */
			sign = 0;
			if(*character == '-')
			{
				sign = -1;
				character++;
			}
			else if(*character == '+')
			{
				sign = +1;
				character++;
			}

			/* %num or others */
//			character++;
			digit1 = 0;
			while(('0' <= *character) && (*character <= '9'))
			{
				digit1 = digit1 * 10 + *character - '0';
				character++;
			}

			/* %.num or others */
			digit2 = -1;
			if(*character == '.')
			{
				character++;
				digit2 = 0;
				while(('0' <= *character) && (*character <= '9'))
				{
					digit2 = digit2 * 10 + *character - '0';
					character++;
				}
			}

			switch(*character)
			{
			case 'c':
			case 'C':
				byte = va_arg(list, char);
				size = ChangeFromC(pOutput, size, 32767,  byte, sign, digit1);
				break;

			case '%':
				pOutput[size] = '%';
				pOutput[++size] = '\0';
				break;

			case 's':
			case 'S':
				pString = va_arg(list, char*);
				size = ChangeFromS(pOutput, size, 32767, pString, sign, digit1, digit2);
				break;

			case 'd':
			case 'D':
				number = va_arg(list, long);
				size = ChangeFromD(pOutput, size, 32767, number, sign, digit1, digit2, 10);
				break;

			case 'x':
			case 'X':
				number = va_arg(list, long);
				size = ChangeFromX(pOutput, size, 32767, number, sign, digit1, digit2, 16, *character);
				break;

			case 'p':
			case 'P':
				number = va_arg(list, long);
				size = ChangeFromP(pOutput, size, 32767, number, sign, digit1, -1, 16);
				break;

			default:
				break;
			}
			character++; /* next character */
		}
	}

	return size;
}
#endif  /* PAFEE_LIB_STDIO_ENABLE */

/**
 * @brief  stdio.snprintf function
 * @param  convert string
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
int pafee_snprintf(
	char *convertString,
	const unsigned int maxSize,
	const char *pFormat,
	...
)
{
	va_list list;
	const char *character;
	char *pString;
	char *pOutput;
	int size;
	int maxSize_tmp;
	long number;
	char byte;
	int sign;
	int digit1;
	int digit2;

	va_start(list, pFormat);
	pOutput = convertString;
	character = pFormat;
	size = 0;

	/* Write max length */
	if(maxSize > 32767)
	{
		maxSize_tmp = 32767;
	}
	else
	{
		maxSize_tmp = (int)maxSize;
	}

	/* writing roop */
	while((*character != 0) && (size <= maxSize_tmp) && (size > -1))
	{
		if(*character != '%') /* if not % */
		{
			if(size >= maxSize_tmp)
			{
				return (-1);
			}
			pOutput[size++] = *character;
			if(size < maxSize_tmp)
			{
				pOutput[size] = '\0';
			}

			character++; /* next character */
		}
		else /* % */
		{
			character++; /* next character */

			/* %+ or %- or others */
			sign = 0;
			if(*character == '-')
			{
				sign = -1;
				character++;
			}
			else if(*character == '+')
			{
				sign = +1;
				character++;
			}

			/* %num or others */
//			character++;
			digit1 = 0;
			while(('0' <= *character) && (*character <= '9'))
			{
				digit1 = digit1 * 10 + *character - '0';
				character++;
			}

			/* %.num or others */
			digit2 = -1;
			if(*character == '.')
			{
				character++;
				digit2 = 0;
				while(('0' <= *character) && (*character <= '9'))
				{
					digit2 = digit2 * 10 + *character - '0';
					character++;
				}
			}

			switch(*character)
			{
			case 'c':
			case 'C':
				byte = va_arg(list, char);
				size = ChangeFromC(pOutput, size, maxSize_tmp, byte, sign, digit1);
				break;

			case '%':
				pOutput[size++] = '%';
				if(size < maxSize_tmp)
				{
					pOutput[size] = '\0';
				}
				else
				{
					size = -1;
				}
				break;

			case 's':
			case 'S':
				pString = va_arg(list, char*);
				size = ChangeFromS(pOutput, size, maxSize_tmp, pString, sign, digit1, digit2);
				break;

			case 'd':
			case 'D':
				number = va_arg(list, long);
				size = ChangeFromD(pOutput, size, maxSize_tmp, number, sign, digit1, digit2, 10);
				break;

			case 'x':
			case 'X':
				number = va_arg(list, long);
				size = ChangeFromX(pOutput, size, maxSize_tmp, number, sign, digit1, digit2, 16, *character);
				break;

			case 'p':
			case 'P':
				number = va_arg(list, long);
				size = ChangeFromP(pOutput, size, maxSize_tmp, number, sign, digit1, digit2, 16);
				break;

			default:
				break;
			}
			character++;

		}
	}

	return size;
}

/*********************************************************/
/*                                                       */
/* static function                                       */
/*                                                       */
/*********************************************************/
/**
 * @brief  %s result
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int ChangeFromS(
	char *pOutputString,         /**< output string */
	const int length,            /**< length of string */
	const int maxLength,         /**< limit length of string */
	const char *pInputString,    /**< %s string */
	const int sign,              /**< %- or %+ */
	const int digit1,            /**< %num non = -1 */
	const int digit2             /**< %.num non = -1*/
)
{
	char *pOutput;
	int inputSize;
	int spaceSize;
	int limitSize;
	register int size;
	register int loop;
	register int count;

	pOutput = pOutputString;
	size = 0;

	/* Input string size */
	inputSize = 0;
	while(pInputString[inputSize] != '\0')
	{
		inputSize++;
	}

	/* Output string length limit */
	if(digit2 < 0)
	{
		limitSize = 0;
	}
	else
	{
		limitSize = digit2;
	}

	if(limitSize != 0)
	{
		if(limitSize < inputSize)
		{
			inputSize = limitSize;
			limitSize = 0;
		}
	}

	/* space size */
	spaceSize = digit1 - inputSize;
	if(spaceSize < 0)
	{
		spaceSize = 0;
	}

	if(sign >= 0)
	{
		loop = 0;

		/* fill space */
		count = length + size;
		while(size < spaceSize)
		{
			if(count >= maxLength)
			{
				return (-1);
			}
			pOutput[count++] = ' ';
			size++;
		}

		/* fill string */
		count = length + spaceSize;
		while(size < (spaceSize + inputSize))
		{
			if(count >= maxLength)
			{
				return (-1);
			}
			pOutput[count++] = pInputString[loop++];
			size++;
		}

		count = length + size;
		if(count < maxLength)
		{
			/* last null */
			pOutput[count] = '\0';
		}
	}
	else
	{
		loop = 0;

		/* fill string */
		while(size < inputSize)
		{
			if((length + loop) >= maxLength)
			{
				return (-1);
			}
			pOutput[length + loop] = pInputString[loop];
			size++;
			loop++;
		}

		/* fill space */
		count = length + size;
		while(size < (inputSize + spaceSize))
		{
			if(count >= maxLength)
			{
				return (-1);
			}
			pOutput[count++] = ' ';
			size++;
		}

		if(count < maxLength)
		{
			/* last null */
			pOutput[count] = '\0';
		}
	}

	return count;
}

/**
 * @brief  %x result
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int ChangeFromX(
	char *pOutputString,          /**< output string */
	const int length,             /**< length of string */
	const int maxLength,          /**< limit length of string */
	const long number,            /**< %d integer */
	const int sign,               /**< %- or %+ */
	const int digit1,             /**< %num non = -1 */
	const int digit2,             /**< %.num non = -1*/
	const unsigned char base,     /**< change base */
	const char outCase            /**< case of output character */
)
{
	char *pOutput;
	int inputSize;
	int spaceSize;
	int paddingSize;
	register int size;
	register int loop;
	register int count;
	unsigned long divNumber;
	char inputNumber;
	char isOverBuffer = 0;

	pOutput = pOutputString;
	size = 0;

	if(base > 0)
	{
		/* Input string size */
		inputSize = 0;
		divNumber = number;

		if(divNumber == 0)
		{
			inputSize++;
		}
		else
		{
			while(divNumber)
			{
				divNumber /= base;
				inputSize++;
			}
		}

		/* space size */
		spaceSize = digit1 - inputSize;
		if(spaceSize < 0)
		{
			spaceSize = 0;
		}
		
		/* Limit string size */
		if(digit2 < 0)
		{
			paddingSize = 0;
		}
		else
		{
			paddingSize = digit2;
		}

		if(paddingSize != 0)
		{
			if(paddingSize < inputSize)
			{
				paddingSize = 0;
			}
			else if(paddingSize < (inputSize + spaceSize))
			{
//				inputSize = inputSize;
//				spaceSize = limitSize - inputSize;
				spaceSize = spaceSize + inputSize - paddingSize;
				paddingSize -= inputSize;
			}
			else
			{
				paddingSize -= inputSize;
				spaceSize = 0;
			}
		}

		if(paddingSize < 0)
		{
			paddingSize = 0;
		}
		if(spaceSize < 0)
		{
			spaceSize = 0;
		}

		loop = 0;
		if(sign >= 0)
		{
			/* fill space */
			count = length + size;
			while(size < spaceSize)
			{
				if(count >= maxLength)
				{
					return (-1);
				}
				pOutput[count++] = ' ';
				size++;
			}

			/* fill zero */
			while(size < (spaceSize + paddingSize))
			{
				if(count >= maxLength)
				{
					return (-1);
				}
				pOutput[count++] = '0';
				size++;
			}

			/* fill string */
			divNumber = number;
			count = length + spaceSize + paddingSize + inputSize - 1;
			while(count >= maxLength)
			{
				isOverBuffer = 1;
				divNumber /= base;
				loop++;
				count--;
			}
			while(loop < inputSize)
			{
				inputNumber = (char)(divNumber % base);
				divNumber /= base;

				pOutput[count--] = ChangeIntToHex(inputNumber, outCase);
				size++;
				loop++;
			}
			if(isOverBuffer == 1)
			{
				return (-1);
			}

			if((length + size) < maxLength)
			{
				/* last null */
				pOutput[length + size] = '\0';
			}
		}
		else
		{
			/* fill zero */
			count = length + size;
			while(size < paddingSize)
			{
				if(count >= maxLength)
				{
					return (-1);
				}
				pOutput[count++] = '0';
				size++;
			}

			/* fill string */
			divNumber = number;
			count = length + paddingSize + inputSize - 1;
			while(count >= maxLength)
			{
				isOverBuffer = 1;
				divNumber /= base;
				loop++;
				count--;
			}
			while(loop < inputSize)
			{
				inputNumber = (char)(divNumber % base);
				divNumber /= base;

				pOutput[count--] = ChangeIntToHex(inputNumber, outCase);
				size++;
				loop++;
			}
			if(isOverBuffer == 1)
			{
				return (-1);
			}

			/* fill space */
			while(size < spaceSize + paddingSize + inputSize)
			{
				if((length + size) >= maxLength)
				{
					return (-1);
				}
				pOutput[length + size] = ' ';
				size++;
			}

			if((length + size) < maxLength)
			{
				/* last null */
				pOutput[length + size] = '\0';
			}
		}
	}

	return (length + size);
}

/**
 * @brief  %c result
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int ChangeFromC(
	char *pOutputString,     /**< output string */
	const int length,        /**< length of string */
	const int maxLength,     /**< limit length of string */
	const char input,        /**< %c character */
	const int sign,          /**< %- or %+ */
	const int digit1         /**< %num non = -1 */
)
{
	char *pOutput;
	int spaceSize;
	register int size;
	register int count;

	pOutput = pOutputString;

	/* space size */
	spaceSize = digit1 - 1;
	if(spaceSize < 0)
	{
		spaceSize = 0;
	}

	size = 0;
	count = length + size;

	if(sign >= 0)
	{
		/* fill space */
		while(size < spaceSize)
		{
			if(count >= maxLength)
			{
				return (-1);
			}
			pOutput[count++] = ' ';
			size++;
		}

		/* fill string */
		if(count >= maxLength)
		{
			return (-1);
		}
		pOutput[count++] = input;
		size++;

		if(count < maxLength)
		{
			/* last null */
			pOutput[count] = '\0';
		}
	}
	else
	{
		/* fill string */
		if(count >= maxLength)
		{
			return (-1);
		}
		pOutput[count++] = input;
		size++;

		/* fill space */
		while(size < spaceSize + 1)
		{
			if(count >= maxLength)
			{
				return (-1);
			}
			pOutput[count++] = ' ';
			size++;
		}

		if(count < maxLength)
		{
			/* last null */
			pOutput[count] = '\0';
		}
	}

	return count;
}

/**
 * @brief  %d result
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int ChangeFromD(
	char *pOutputString,      /**< output string */
	const int length,         /**< length of string */
	const int maxLength,      /**< limit length of string */
	const long number,        /**< %d integer */
	const int sign,           /**< %- or %+ */
	const int digit1,         /**< %num non = -1 */
	const int digit2,         /**< %.num non = -1*/
	const unsigned char base  /**< change base */
)
{
	char *pOutput;
	int inputSize;
	int spaceSize;
	int paddingSize;
	register int size;
	register int loop;
	register int count;
	long divNumber;
	int signSize;
	char inputNumber;
	char isOverBuffer = 0;

	pOutput = pOutputString;
	size = 0;

	if(base > 0)
	{
		/* Input string size */
		inputSize = 0;
		divNumber = number;

		if(divNumber == 0)
		{
			inputSize++;
		}
		else
		{
			if(divNumber < 0)
			{
				divNumber = -divNumber;
			}
			while(divNumber)
			{
				divNumber /= base;
				inputSize++;
			}
		}

		/* space size */
		spaceSize = digit1 - inputSize;
		if(spaceSize < 0)
		{
			spaceSize = 0;
		}

		/* Limit string size */
		if(digit2 < 0)
		{
			paddingSize = 0;
		}
		else
		{
			paddingSize = digit2;
		}

		if(paddingSize != 0)
		{
			if(paddingSize < inputSize)
			{
				paddingSize = 0;
			}
			else if(paddingSize < (inputSize + spaceSize))
			{
				spaceSize = spaceSize + inputSize - paddingSize;
				paddingSize -= inputSize;
			}
			else
			{
				paddingSize -= inputSize;
				spaceSize = 0;
			}
		}

		if(paddingSize < 0)
		{
			paddingSize = 0;
		}
		if(spaceSize < 0)
		{
			spaceSize = 0;
		}

		if(sign >= 0)
		{
			/* fill space */
			count = length + size;
			while(size < spaceSize)
			{
				if(count >= maxLength)
				{
					return (-1);
				}
				pOutput[count++] = ' ';
				size++;
			}

			divNumber = number;
			loop = 0;

			/* insert sign */
			if(divNumber < 0)
			{
				divNumber = -divNumber;
				if(spaceSize == 0)
				{
					if((length + size) >= maxLength)
					{
						return (-1);
					}
					pOutput[length + spaceSize] = '-';
					spaceSize++;
					size++;

				}
				else
				{
					pOutput[length + spaceSize - 1] = '-';
				}
			}
			else if(sign > 0)
			{
				if(spaceSize == 0)
				{
					if((length + size) >= maxLength)
					{
						return (-1);
					}
					pOutput[length + spaceSize] = '+';
					spaceSize++;
					size++;
				}
				else
				{
					pOutput[length + spaceSize - 1] = '+';
				}
			}

			/* fill zero */
			while(size < (spaceSize + paddingSize))
			{
				if((length + size) >= maxLength)
				{
					return (-1);
				}
				pOutput[length + size] = '0';
				size++;
			}

			/* fill string */
			count = length + spaceSize + paddingSize + inputSize - 1;
			while(count >= maxLength)
			{
				isOverBuffer = 1;
				divNumber /= base;
				loop++;
				count--;
			}
			while(loop < inputSize)
			{
				inputNumber = (char)(divNumber % base);
				divNumber /= base;

				pOutput[count--] = ChangeIntToHex(inputNumber, 'd');
				size++;
				loop++;
			}
			if(isOverBuffer == 1)
			{
				return (-1);
			}

			count = length + size;
			if(count < maxLength)
			{
				/* last null */
				pOutput[count] = '\0';
			}
		}
		else
		{
			divNumber = number;
			loop = 0;
			signSize = 0;

			/* insert sign */
			if(divNumber < 0)
			{
				divNumber = -divNumber;
				pOutput[length] = '-';
				if(spaceSize > 0)
				{
					spaceSize--;
				}
				size++;
				if((length + size) >= maxLength)
				{
					return (-1);
				}
				signSize++;
			}
			else if(sign > 0)
			{
				if((length + size) >= maxLength)
				{
					return (-1);
				}
				pOutput[length] = '+';
				if(spaceSize > 0)
				{
					spaceSize--;
				}
				size++;
				signSize++;
			}

			/* fill zero */
			count = length + size;
			while(size < (paddingSize + signSize))
			{
				if(count >= maxLength)
				{
					return (-1);
				}
				pOutput[count++] = '0';
				size++;
			}

			/* fill string */
			count = length + signSize + paddingSize + inputSize - 1;
			while(count >= maxLength)
			{
				isOverBuffer = 1;
				divNumber /= base;
				loop++;
				count--;
			}
			while(loop < inputSize)
			{
				inputNumber = (char)(divNumber % base);
				divNumber /= base;

				pOutput[count--] = ChangeIntToHex(inputNumber, 'd');
				size++;
				loop++;
			}
			if(isOverBuffer == 1)
			{
				return (-1);
			}

			count = length + size;
			/* fill space */
			while(size < spaceSize + signSize + paddingSize + inputSize)
			{
				if(count >= maxLength)
				{
					return (-1);
				}
				pOutput[count++] = ' ';
				size++;
			}

			if(count < maxLength)
			{
				/* last null */
				pOutput[count] = '\0';
			}
		}
	}

	return count;
}

/**
 * @brief  %p result
 * @return result
 * @retval if false result=0
 * @retval if true  result=num
 */
static int ChangeFromP(
	char *pOutputString,      /**< output string */
	const int length,         /**< length of string */
	const int maxLength,      /**< limit length of string */
	const long number,        /**< %d integer */
	const int sign,           /**< %- or %+ */
	const int digit1,         /**< %num non = -1 */
	const int digit2,         /**< %.num non = -1 */
	const unsigned char base  /**< change base */
)
{
	char *pOutput;
	int inputSize;
	int spaceSize;
	register int size;
	register int loop;
	register int count;
	long divNumber;
	char inputNumber;
	char isOverBuffer = 0;

	pOutput = pOutputString;
	size = 0;

	if(base > 0)
	{
		/* Input string size */
		inputSize = 8;

		/* space size */
		spaceSize = digit1 - inputSize;
		if(spaceSize < 0)
		{
			spaceSize = 0;
		}

		loop = 0;
		divNumber = number;
		if(sign >= 0)
		{
			/* fill space */
			count = length + size;
			while(size < spaceSize)
			{
				if(count >= maxLength)
				{
					return (-1);
				}
				pOutput[count++] = ' ';
				size++;
			}

			/* fill string */
			count = length + spaceSize + inputSize - 1;
			while(count >= maxLength)
			{
				isOverBuffer = 1;
				divNumber /= base;
				loop++;
				count--;
			}
			while(loop < inputSize)
			{
				inputNumber = (char)(divNumber % base);
				divNumber /= base;

				pOutput[count--] = ChangeIntToHex(inputNumber, 'X');  /* upper case */
				size++;
				loop++;
			}
			if(isOverBuffer == 1)
			{
				return (-1);
			}

			count = length + size;
			if(count < maxLength)
			{
				/* last null */
				pOutput[count] = '\0';
			}
		}
		else
		{
			/* fill string */
			count = length + inputSize - 1;
			while(count >= maxLength)
			{
				isOverBuffer = 1;
				divNumber /= base;
				loop++;
				count--;
			}
			while(loop < inputSize)
			{
				inputNumber = (char)(divNumber % base);
				divNumber /= base;

				pOutput[count--] = ChangeIntToHex(inputNumber, 'X');  /* upper case */
				size++;
				loop++;
			}
			if(isOverBuffer == 1)
			{
				return (-1);
			}

			count = length + size;
			/* fill space */
			while(size < (inputSize + spaceSize))
			{
				if(count >= maxLength)
				{
					return (-1);
				}
				pOutput[count++] = ' ';
				size++;
			}

			if(count < maxLength)
			{
				/* last null */
				pOutput[count] = '\0';
			}
		}

	}

	return count;
}

/**
 * @brief  int to hex
 * @return result
 * @retval if false result=0
 * @retval if true  result=hex character
 */
static char ChangeIntToHex(
	char digit,    /**< hex number */
	char outCase   /**< case of output character */
)
{
	if((0 <= digit) && (digit <= 9))
	{
		return (char)('0' + digit);
	}
	else if((10 <= digit) && (digit <= 35))
	{
		if(outCase == 'X')
		{
			return (char)('A' + digit - 10);
		}
		else
		{
			return (char)('a' + digit - 10);
		}
	}
	else
	{
		return 0;
	}
}

