//****************************************************
//
//	project	PAFEE
//!	@file:	LockFreeRingBuffer.h
//!	@author: 
//!	@brief:	
//
//****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _LOCK_FREE_RING_BUFFER_H_INCLUDED_
#define _LOCK_FREE_RING_BUFFER_H_INCLUDED_

#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "assert.h"
#include "pafee_string.h"

namespace Pafee
{

class LockFreeRingBuffer : public Object
{
#ifdef PAFEE_STATISTICS_ENABLE
public:
	typedef struct
	{
		int	eneqeueCount;		;///< enqueue request count
		int	overflowCount;		;///< overflow count 
		int	peakElementCount;	;///< peak queue element count
		int	currentElementCount;;///< current queue element count
	} STATISTICS;
#endif // PAFEE_STATISTICS_ENABLE

public:
	LockFreeRingBuffer(int capacity=0, int elementSize=0, Allocator* pAllocator=NULL);

	~LockFreeRingBuffer();

public:
	bool Initialize(int capacity, int elementSize, Allocator* pAllocator=NULL);

	int GetCount(void);

	bool IsEmpty(void);

	void* GetWriteBuffer(int* pLength);

	bool Enqueue(void* pData, int length);
	bool Enqueue(int length);

	bool Dequeue(void* pBuffer, int* pLength);
	void* Dequeue(int* pLength);

	bool Peek(void* pBuffer, int* pLength);
	void* Peek(int* pLength=NULL);

	int PeekLength(void);

	void ClearAll(void);

#ifdef PAFEE_STATISTICS_ENABLE
	void GetStatistics(LockFreeRingBuffer::STATISTICS* pStatistics);
#endif

protected:
	//! buffer element
	typedef struct
	{
		char* 			pData;		///< data buffer
		int 	length;		///< data length
	} BUFFER_ELEMENT;
	
protected:
	char*			m_pBuffer;			///< buffer pointer for copy-avoidance method
	BUFFER_ELEMENT*	m_pElementArray;	///< ring buffer elements

	int m_bufferCapacity;			///< buffer capacity counts
	int m_maxElementSize;			///< max element size
	int m_readPoint;				///< read point index
	int m_writePoint;				///< write point index
	unsigned int m_totalReadCount;	///< total read count
	unsigned int m_totalWriteCount;	///< total write count
	bool m_isInitialized;			///< initialized flag

#ifdef PAFEE_STATISTICS_ENABLE
	STATISTICS m_statistics;		///< statistics
#endif
#ifdef PAFEE_UNITTEST
public:
	static void UnitTest(void);
#endif
};

}; // namespace Pafee

#endif // _LOCK_FREE_RING_BUFFER_H_INCLUDED_

