//****************************************************
//
//    project   Pafee
//!   @file:    PafeePriorityQueue.h
//!   @author:  M.Mori
//!   @brief:   PafeePriorityQueuea Class
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _INCLUDE_PRIORITYQUEUE_H_
#define _INCLUDE_PRIORITYQUEUE_H_
#include <new>
#include "PafeeObject.h"
#include "PafeeAllocator.h"
#include "PafeeList.h"

namespace Pafee
{

//! PriorityQueue
class PriorityQueue : public Object
{
private:
	inline int AligmentSize(int size) {return (((size-1)|((size-1)^3))+1);};
public:
	PriorityQueue( int priorityRange=0, Allocator* pAllocator=NULL);
	PriorityQueue( const PriorityQueue& pquec, Allocator* pAllocator=NULL );
	~ PriorityQueue();
	PriorityQueue& operator =( const PriorityQueue& pquec );
	bool Initialize(int priorityRange, Allocator* pAllocator=NULL);
	int GetCount()
	{
		int rtn=0;
		for(int i=0; i<m_priorityRange; i++ )
		{
			int rtn2 = m_pPriorityTable[i].GetCount();
			rtn +=rtn2;
		}
		return rtn;
	}
	bool IsEmpty(void)
	{
		int rtn=0;
		for(int i=0; i<m_priorityRange; i++ )
		{
			int rtn2 = m_pPriorityTable[i].GetCount();
			rtn +=rtn2;
		}
		return rtn==0;
	}
	void* GetWriteBuffer(int priority, int length);
	bool Enqueue (int priority, void* pData, int length);
	bool Enqueue (int priority, int length);
	bool Dequeue(int* pPriority, void* pBuffer, int* pLength, int bufSize=INT_MAX);
	bool Peek(int* pPriority, void* pBuffer, int* pLength, int bufSize=INT_MAX);
	void* Peek (int* pPriority=NULL, int* pLength=NULL);
	int PeekLength(int* pPriority=NULL);
	void RemoveFront(void);
	void RemoveAll(void);

#ifdef PAFEE_STATISTICS_ENABLE
	typedef struct
	{
		int addCount;
		int maxElementCount;
		int minElementSize;
		int maxElementSize;
		int presentElementCount;
	} STATISTICS;
	void GetStatistics(PriorityQueue::STATISTICS* pStatistics);
	PriorityQueue::STATISTICS m_Statistics;
#endif //PAFEE_STATISTICS_ENABLE

#ifdef PAFEE_UNITTEST
	static void PriorityQueue::UnitTest(void);
#endif //PAFEE_UNITTEST

private:
	List *m_pPriorityTable;
	char* m_pWriteBuffer;
	int m_priorityRange;
};

}

#endif

