//****************************************************
//
//    project   Pafee
//!   @file:    PafeeQueueT.h
//!   @author:  M.Tanaka
//!   @brief:   QueueT Hedder File
//
//*****************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Systems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _PAFEE_QUEUET_H_INCLUDED_
#define _PAFEE_QUEUET_H_INCLUDED_

#include "PafeeCommon.h"
#include "PafeeObject.h"
#include "PafeeListT.h"
#include "pafee_string.h"
#include <limits.h>

#ifdef PAFEE_UNITTEST
#include <assert.h>
#endif //PAFEE_UNITTEST

namespace Pafee
{

//! QueueT
template<class T>
class QueueT : public Object
{

#ifdef PAFEE_STATISTICS_ENABLE
public:
	typedef struct
	{
		int addCount;
		int maxElementCount;
		int presentElementCount;
	} STATISTICS;

private:
	STATISTICS m_Statistics;
#endif // PAFEE_STATISTICS_ENABLE

public:
	QueueT(
		Allocator* pAllocator=NULL ///< [in] Memory Allocator
	)
	{
		SetAllocator(pAllocator);
		m_pList = (ListT<T>*)Allocate(sizeof(ListT<T>));
		m_pList = new (m_pList)ListT<T>;
	}
	QueueT(
		const QueueT& quec, ///< [in] initial value of Queue
		Allocator* pAllocator=NULL ///< [in] Memory Allocator
	)
	{
		SetAllocator(pAllocator);
		m_pList = (ListT<T>*)Allocate(sizeof( ListT<T>));
		new (m_pList)ListT<T>(pAllocator);
		*m_pList = *quec.m_pList;
	}

	virtual ~QueueT()
	{
		m_pList->~ListT<T>();
		Deallocate(m_pList);
	}

	QueueT& operator =(
		const QueueT& quec ///< [in] Copied QueueT
	)
	{
		*m_pList = *quec.m_pList;
		return *this;
	}

	int GetCount(void)
	{
		return m_pList->GetCount();
	}

	bool IsEmpty(void)
	{
		return m_pList->IsEmpty();
	}

	bool Enqueue(
		const T& data ///[in] Input data
	)
	{
		return m_pList->AddBack(data);
	}
	bool Dequeue(
		T& data ///[out] Output data
	)
	{
		if(IsEmpty())
		{
			return false;
		}
		data = m_pList->GetFront();
		if(m_pList->RemoveFront())
		{
			return true;
		}
		return false;
	}

	T& Peek(void)
	{
		return m_pList->GetFront();
	}

	void RemoveFront(void)
	{
		m_pList->RemoveFront();
	}

	void RemoveAll(void)
	{
		m_pList->RemoveAll();
	}

#ifdef PAFEE_STATISTICS_ENABLE
	void GetStatistics(STATISTICS *pStatistics)
	{
		struct ListT<T>::STATISTICS  statisticsRtn;
		m_pList->GetStatistics(&statisticsRtn);
		m_Statistics.addCount = statisticsRtn.addCount;
		m_Statistics.maxElementCount = statisticsRtn.maxElementCount;
		m_Statistics.presentElementCount = statisticsRtn.presentElementCount;
//		pafee_memcpy ( pStatistics, &m_Statistics, sizeof (struct QueueT<T>::STATISTICS) );
		pafee_memcpy ( pStatistics, &m_Statistics, sizeof m_Statistics );
	}
#endif //PAFEE_STATISTICS_ENABLE

protected:
	ListT<T>* m_pList;

};

};

#endif

