//********************************************************************
//
//! 	 @file: 	PafeeCommon.h
//! 	 @author:	T.Miyawaki
//! 	 @brief:	common defines and macros
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _PAFEE_COMMON_H_INCLUDED_
#define _PAFEE_COMMON_H_INCLUDED_

#ifndef NULL
#define NULL (0)
#endif

#ifdef PAFEE_STATISTICS_ENABLE
#include <memory.h>
#define STATISTICS_INC(DATA)		(DATA)++
#define STATISTICS_DEC(DATA)		(DATA)--
#define STATISTICS_ADD(DATA, VALUE)	(DATA) += (VALUE)
#define STATISTICS_SUB(DATA, VALUE)	(DATA) -= (VALUE)
#define STATISTICS_SET(DATA,VALUE)	(DATA) = (VALUE)
#define STATISTICS_MAX(DATA, VALUE)	if((VALUE) > (DATA)){ (DATA) = (VALUE); }
#define STATISTICS_MIN(DATA, VALUE)	if((VALUE) < (DATA)){ (DATA) = (VALUE); }
#else
#define STATISTICS_INC(DATA)
#define STATISTICS_DEC(DATA)
#define STATISTICS_ADD(DATA, VALUE)
#define STATISTICS_SUB(DATA, VALUE)
#define STATISTICS_SET(DATA,VALUE)
#define STATISTICS_MAX(DATA, VALUE)
#define STATISTICS_MIN(DATA, VALUE)
#endif


#ifdef PAFEE_DEBUG
#include <stdio.h>
#include <assert.h>
#define PAFEE_ASSERT(CONDITION)		assert((CONDITION));
#else
#define PAFEE_ASSERT(CONDITION)
#endif


#ifdef PAFEE_UNITTEST
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <memory.h>
#endif


#ifdef PAFEE_EXCEPTION
#include <new>
#include <exception>
#include <stdexcept>
#define THROW(EXCEPTION)	throw EXCEPTION;
#else
#define THROW(EXCEPTION)
#endif


#endif // _PAFEE_COMMON_H_INCLUDED_



