//********************************************************************
//		project 	PAFEE
//
//! 	 @file: 	PafeeQueuedListener.h
//! 	 @author:	Takatsura Miyawaki
//! 	 @brief:	Event Listener with queue
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _EVENT_LISTENER_H_INCLUDED_
#define _EVENT_LISTENER_H_INCLUDED_

#include "PafeeCommon.h"
#include "PafeeSignal.h"
#include "PafeeMutex.h"
#include "PafeeTask.h"
#include "PafeeList.h"
#include "PafeeListener.h"

namespace Pafee
{

//! Event listener with queue
class QueuedListener : public Listener, public PafeeTask
{

public:

	//! Constructor
	QueuedListener();

	//! Desstructor
	virtual ~QueuedListener();

	//! Intialize (for TOPPERS)
#ifdef PAFEE_OS_ITRON
	bool Initialize(
					ID taskID,					///< [in] task ID
					ID signalID,				///< [in] signal ID
					ID mutexID,					///< [in] mutex ID
					Allocator* pAllocator=NULL	///< [in] allocator for queue
					);
#else
	//! Intialize (for Windows/Linux)
	bool Initialize(
					TASK_PRIORITY priority,			///< [in] task priority
					TASK_SCHEDULING schedule = 0,	///< [in] task scheduling
					int stackSize = 0,				///< [in] stack size
					Allocator* pAllocator=NULL		///< [in] allocator for queue
					);
#endif

	// Terminate 
	void Terminate(void);

	//! Send event
	virtual bool SendEvent(
							unsigned long eventID,	///< [in] eventID
							const void* pParam,		///< [in] pointer of event parameter
							int paramLength			///< [in] event parameter length
							);

protected:

	//! Task procedure
	virtual void* TaskProcedure(
							void* pParam	///< [in] task parameter
							);

protected:
	TASK_ID     m_taskID;		///< task ID
	PafeeSignal	m_signal;		///< signal
	PafeeMutex	m_mutex;		///< mutex
	List		m_queue;		///< queue for event
	bool		m_isKeepRunning;///< task run control flag
	bool		m_isRunning;	///< task running flag
};

};

#endif // _EVENT_LISTENER_H_INCLUDED_
