//********************************************************************
//
//! 	 @file: 	PafeeAllocator.h
//! 	 @author:	T.Miyawaki
//! 	 @brief:	Memory allocator
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef _ALLOCATOR_H_INCLUDED_
#define _ALLOCATOR_H_INCLUDED_

#include <cstddef>
#ifdef PAFEE_STATISTICS_ENABLE
#include "PafeeStreamOutput.h"
#endif

namespace Pafee {

class Allocator
{

public:
	//! Constractor
	Allocator()
	{ };
	

	//! Destoractor
	virtual ~Allocator()
	{ };
	

	//! Allocate memory
	virtual void* Allocate(unsigned long size) = 0;

	//! Deallocate memory
	virtual void Deallocate(void* ptr) = 0;

	//! Check given address is in own address range
	virtual bool IsOwnAddress(void* ptr) = 0;

#ifdef PAFEE_STATISTICS_ENABLE
	//! Get statistics
	virtual void GetStatistics(StreamOutput& stream){};
#endif

	//! Adjust to 4bytes alignment
	inline static unsigned long AlignUp(unsigned long data)
	{
		if(data & 0x3)
		{
			data &= (~0x3L);
			data += 0x4L;
		}
		return data;
	};
	
	//! Adjust to 4bytes alignment
	inline static unsigned long AlignDown(unsigned long data)
	{
		return (data & (~0x3L));
	}

protected:

private:
	//! Copy constractor
	Allocator(const Allocator& src);

	//! Operator equal
	Allocator& operator=(const Allocator& src);
};

};

#endif // _ALLOCATOR_H_INCLUDED_
