//****************************************************
//
//      project     PAFEE
//!      @file:     CommandOption.h
//!      @author:   Technocrats
//!      @brief:    Command-Line Option Analizer class
//
//*****************************************************

#ifndef _COMMAND_OPTION_H_INCLUDED_
#define _COMMAND_OPTION_H_INCLUDED_

#include <string>
#include "define.h"
#include "assert.h"

using namespace std;

namespace Pafee
{

class CommandOption
{

public:
	/**
	 * @brief	constractor of CommandOption Class
	 */
	CommandOption(const int argc = 0, const char* argv[] = NULL);

	/**
	 * @brief	destoractor of CommandOption Class
	 */
	~CommandOption();

	/**
	 * @brief  Get information from command line option
	 * @param  argc : command line parameter
	 * @param  argv[] : command line parameter
	 * @return Analysis success or not
	 * @retval TRUE Analysis success
	 * @retval FALSE Analysis failure
	 */
	BOOL AnalyzeOption(int argc, char* argv[]);

	/**
	 * @brief  Return Config File Path
	 * @param  (none)
	 * @return Config file path
	 * @retval Config file path
	 */
	string GetConfigFilePath(void);

	/**
	 * @brief  Return Dependency File Path
	 * @param  (none)
	 * @return Dependency file path
	 * @retval Dependency file path
	 */
	string GetDependencyFilePath(void);

	/**
	 * @brief  Return Source Tree Folder Path
	 * @param  (none)
	 * @return Source Tree Folder Path
	 * @retval Source Tree Folder Path
	 */
	string GetSourceTreePath(void);

	/**
	 * @brief  Return Output Folder Path
	 * @param  (none)
	 * @return Output Folder Path
	 * @retval Output Folder Path
	 */
	string GetOutputPath(void);

	/**
	 * @brief  Return whether Print Version information
	 * @param  (none)
	 * @return Print Version information or Not
	 * @retval TRUE : Print Version information
	 * @retval FALSE : Don't Print Version Information
	 */
	BOOL IsVersion(void);

	/**
	 * @brief  Return whether Print Help message
	 * @param  (none)
	 * @return Print Help message or Not
	 * @retval TRUE : Print Help message
	 * @retval FALSE : Don't Print Help message
	 */
	BOOL IsHelp(void);

	/**
	 * @brief  Return whether Use Library for Freestanding Environment
	 * @param  (none)
	 * @return Use Library for Freestanding Environment or Not
	 * @retval TRUE : Use Library for Freestanding Environment
	 * @retval FALSE : Don't Use Library for Freestanding Environment
	 */
	BOOL IsFreestanding(void);

	/**
	 * @brief  Return whether command line include wrong input
	 * @param  (none)
	 * @return command line include wrong input or Not
	 * @retval TRUE : command line include wrong input
	 * @retval FALSE : command line doesn't include wrong input
	 */
	BOOL IsWrongInput(void);

private:
	string m_configFilePath;
	string m_dependencyFilePath;
	string m_sourceTreePath;
	string m_outputPath;
	BOOL m_isVersion;
	BOOL m_isHelp;
	BOOL m_isFreestanding;
	BOOL m_isWrongInput;

};
};	// namespace Pafee
#endif	// _COMMAND_OPTION_H_INCLUDED_
