//****************************************************
//
//      project     PAFEE
//!      @file:     StructureDefinition.h
//!      @author:   Technocrats
//!      @brief:    Structure Definition File Analizer class
//
//*****************************************************

#ifndef _STRUCTURE_DEFINITON_H_INCLUDED_
#define _STRUCTURE_DEFINITON_H_INCLUDED_

#include <string>
#include <list>
#include "define.h"
#include "assert.h"

using namespace std;

namespace Pafee
{

class StructureDefinition
{

public:
	/**
	 * @brief	constractor of StructureDefinition Class
	 */
	StructureDefinition(const string FilePath = "");

	/**
	 * @brief	destoractor of StructureDefinition Class
	 */
	~StructureDefinition();

	/**
	 * @brief  Read Structure Definition File
	 * @param  FilePath : Structure Definition File Path
	 * @return Read success or not
	 * @retval TRUE : Read success
	 * @retval FALSE : Read failure
	 */
	BOOL ReadFile(const string filePath = "");

	
	/**
	 * @brief  Analyze Structure Definition File
	 * @param  (none)
	 * @return Analysis success or not
	 * @retval TRUE : Analysis success
	 * @retval FALSE : Analysis failure
	 */
	BOOL DefineStructure(void);

	/**
	 * @brief  Return Class Information List
	 * @param  pClassTable : Pointer to Class Information List
	 * @return Get success or not
	 * @retval TRUE : Get success
	 * @retval FALSE : Get failure
	 */
	BOOL GetClassList(list<string> *pClassList);

private:
	string m_FilePath;
	list<string> m_usingClassList;

};
};	// namespace Pafee
#endif	// _STRUCTURE_DEFINITON_H_INCLUDED_
