//********************************************************************
//
//! 	 @file: 	PafeeCoverageRecorder.cpp
//! 	 @author:	T.Miyawaki
//! 	 @brief:	Coverage recorder for testing
//
//********************************************************************

//
// P A F E E   - Pragmatic Application Framework for Embedded Environment- 
//
//
// Copyright (c) 2009 KN Information Sustems Corporation,
//                    Hirocon Corporation,
//                    Technocrats Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#include "stdio.h"
#include "PafeeCoverageRecorder.h"

using namespace Pafee;

//--------------------------------------------------------
//! Constractor
//--------------------------------------------------------
CoverageRecorder::CoverageRecorder()
{
	for(int i = 0; i < MAX_COVERAGE; i++)
	{
		m_coverage[i].lineNumber = -1;
		m_coverage[i].count = 0;
	}
	m_dataCount = 0;
	m_overFlow = 0;
}


//--------------------------------------------------------
//! Destoractor
//--------------------------------------------------------
CoverageRecorder::~CoverageRecorder()
{ };

//--------------------------------------------------------
//! Record coverage
//--------------------------------------------------------
void CoverageRecorder::Record(int lineNumber)
{
	for(int i = 0; i < MAX_COVERAGE; i++)
	{
		if(m_coverage[i].lineNumber == lineNumber)
		{
			m_coverage[i].count++;
			return;
		}
		else if(m_coverage[i].lineNumber == -1)
		{
			m_coverage[i].lineNumber = lineNumber;
			m_coverage[i].count++;
			m_dataCount = i+1;
			return;
		}
	}
	m_overFlow++;
}

//--------------------------------------------------------
//! Show coverage
//--------------------------------------------------------
void CoverageRecorder::Show(void)
{
	SortData();
	
	printf("line   passed count\n");
	printf("-------------------\n");
	for(int i = 0; i < m_dataCount; i++)
	{
		printf("[%04d]   %d\n", m_coverage[i].lineNumber, m_coverage[i].count);
	}
	if(m_overFlow)
	{
		printf("\nXXXXXXXX Couldn't record %d lines!! XXXXXXXX\n", m_overFlow);
	}
}
//////////////////////////////////////////////////////////////////////////////////////////
// Protected members

//--------------------------------------------------------
//! Sort data
//--------------------------------------------------------
void CoverageRecorder::SortData(void)
{
	// Bubble sort, it's slow, but no need for extra memory.
	for(int i = 0; i < m_dataCount-1; i++)
	{
		for(int j = m_dataCount-1; j > i ; j--)
		{
			if(m_coverage[j].lineNumber < m_coverage[j-1].lineNumber)
			{
				Swap(m_coverage[j].lineNumber, m_coverage[j-1].lineNumber);
				Swap(m_coverage[j].count, m_coverage[j-1].count);
			}
		}
	}
}

//--------------------------------------------------------
//! Swap data
//--------------------------------------------------------
void CoverageRecorder::Swap(int& dataA, int& dataB)
{
	int temp;
	temp = dataA;
	dataA = dataB;
	dataB = temp;
};




